#!/usr/bin/env python


import httplib
import random
import re

from common import version
from common import Common


class OptionCheck(Common):
    def __defaultUserAgent(self):
        return "sqlmap/%s (http://sqlmap.sourceforge.net)" % version


    def __setHttpHeaders(self):
        """
        Extract random User-Agent HTTP header from a file and
        return into a dictionary
        """

        logMsg  = "fetching random http user agent header from "
        logMsg += "file '%s'" % self.args.userAgentsFile
        self.log(logMsg)

        try:
            fd = open(self.args.userAgentsFile)
        except:
            warnMsg  = "unable to read http user agent header "
            warnMsg += "file '%s'" % self.args.userAgentsFile
            self.warn(warnMsg)

            self.args.httpHeaders["User-Agent"] = self.__defaultUserAgent()

            return

        count = 0
        userAgents = []

        while True:
            line = fd.readline()
            if not line:
                break
            userAgents.append(line)
            count += 1

        fd.close()

        userAgent = userAgents[random.randint(1, count - 1)]
        userAgent = userAgent.replace("\n", "").replace("\r", "")
        self.args.httpHeaders["User-Agent"] = userAgent

        logMsg  = "fetched random http user agent header from "
        logMsg += "file '%s'" % self.args.userAgentsFile
        self.log(logMsg)


    def check(self, shellArgs):
        """
        Perform checks on some command line parameter values
        and set HTTP headers to be used for all HTTP requests
        by self.queryPage() function.
        """

        self.args = shellArgs
        self.args.parameters = {}

        # Perform checks on '--url' parameter value
        if not re.search("^http[s]*://[\w\.\-\_]+", self.args.url, re.I):
            port = re.search("^[\w\.\-\_]+:([\d]+)", self.args.url)
            if port:
                if int(port.groups()[0]) == 443:
                    self.args.url = "https://" + self.args.url
                else:
                    self.args.url = "http://" + self.args.url
            else:
                self.args.url = "http://" + self.args.url

        link = re.search("^http[s]*://([\w\.\-\_]+)", self.args.url, re.I)
        transport = re.search("^(http[s]*)://[\w\.\-\_]+", self.args.url, re.I)
        port = re.search("^http[s]*://[\w\.\-\_]+:([\d]+)", self.args.url, re.I)

        if link:
            self.args.link = link.groups()[0]

        if transport:
            self.args.transport = transport.groups()[0]
        elif port:
            if int(port.groups()[0]) == 443:
                self.args.transport = "https"
            else:
                self.args.transport = "http"
        else:
            self.args.transport = "http"

        if port:
            self.args.port = int(port.groups()[0])
        elif self.args.transport == "http":
            self.args.port = 80
        else:
            self.args.port = 443

        if self.args.transport == "http":
            self.args.http = httplib.HTTPConnection
        else:
            self.args.http = httplib.HTTPSConnection

        # Perform checks on http user agents file (-a) parameter value
        self.args.httpHeaders = {}

        if self.args.userAgentsFile:
            self.__setHttpHeaders()
        else:
            self.args.httpHeaders["User-Agent"] = self.__defaultUserAgent()

        # Perform checks on '--method' parameter value
        if self.args.httpMethod:
            self.args.httpMethod = self.args.httpMethod.upper()

            if self.args.httpMethod not in ("GET", "POST"):
                warnMsg  = "'%s' " % self.args.httpMethod
                warnMsg += "is an unsupported HTTP method, "
                warnMsg += "setting to default method, GET"
                self.warn(warnMsg)

                self.args.httpMethod = "GET"
        else:
            self.args.httpMethod = "GET"

        # Setup the rough parameters string
        if self.args.httpMethod == "GET":
            if "?" in self.args.url:
                url, parameters = self.args.url.split("?")
            else:
                errMsg  = "you did not provide the parameters "
                errMsg += "in the target url"
                raise Exception, errMsg
        elif self.args.httpMethod == "POST":
            # Perform checks on '--data' parameter value
            if self.args.data:
                parameters = self.args.data
            else:
                errMsg = "HTTP POST method depends on '--data' value"
                raise Exception, errMsg

        # Setup the dictionary of parameters to perform SQL injection
        # in the format:
        #     {'paramName': 'paramValue'}
        for element in parameters.split("&"):
            elem = element.split("=")
            if len(elem) == 2:
                parameter = elem[0]

                # Perform checks on testable parameter (-p)
                condition  = not self.args.urlParameter
                condition |= parameter == self.args.urlParameter

                if condition:
                    value = elem[1]
                    if value:
                        self.args.parameters[parameter] = value

        # Finalize target url value
        if self.args.httpMethod == "POST":
            if "?" in self.args.url:
                self.args.url = self.args.url.split("?")[0]
            self.args.url += "?" + parameters

        # Perform checks on '--cookie' parameter value
        if self.args.cookie:
            self.args.httpHeaders["Connection"] = "Keep-Alive"
            self.args.httpHeaders["Cookie"] = self.args.cookie

        # Perform checks on '--proxy' parameter value
        if self.args.proxy:
            if not re.search("[\w\.\-\_]+\:[\w\.\-\_]+", self.args.proxy):
                errMsg  = "proxy value must be in format "
                errMsg += "'url:port'"
                raise Exception, errMsg
            else:
                proxy = self.args.proxy.replace("http://", "")
                proxy = re.search("([\w\.\-\_\:]+)", proxy)
                self.args.proxy = proxy.groups()[0]

        return self.args
