#!/usr/bin/env python


import time

from optparse import OptionParser
from optparse import OptionError

from common import version
from option import OptionCheck
from injection import InjectionCheck
from dump import SQLMapDump
from mysqlmap import MySQLMap
from postgresqlmap import PostgreSQLMap
from mssqlservermap import MSSQLServerMap


def showBanner():
    print """
    sqlmap/%s coded by inquis <bernardo.damele@gmail.com>
                     and belch <daniele.bellucci@gmail.com>
    """ % version


def parseArgs():
    parser = OptionParser()

    try:
        parser.add_option("-u", "--url",
                          dest="url",
                          help="target url")

        parser.add_option("-p",
                          dest="urlParameter",
                          help="specify the injectable parameter")

        parser.add_option("-a",
                          dest="userAgentsFile",
                          help="load http User-Agent random "
                               "header from file")

        parser.add_option("--method",
                          dest="httpMethod",
                          help="http method, GET or POST "
                               "(default: GET)")

        parser.add_option("--data",
                          dest="data",
                          help="data string to be sent through POST")

        parser.add_option("--cookie",
                          dest="cookie",
                          help="http cookie header")

        parser.add_option("--proxy",
                          dest="proxy",
                          help="use a proxy to connect to the url")

        parser.add_option("--string",
                          dest="string",
                          help="string to match in page "
                               "when the query is valid")

        parser.add_option("-f", "--fingerprint",
                          dest="exaustiveFp",
                          action="store_true",
                          help="perform an exaustive database fingerprint")

        parser.add_option("-b", "--banner",
                          dest="getBanner",
                          action="store_true",
                          help="get database banner")

        parser.add_option("--current-user",
                          dest="getCurrentUser",
                          action="store_true",
                          help="get current database user")

        parser.add_option("--current-db",
                          dest="getCurrentDb",
                          action="store_true",
                          help="get current database name")

        parser.add_option("--users",
                          dest="getUsers",
                          action="store_true",
                          help="get database users")

        parser.add_option("--dbs",
                          dest="getDbs",
                          action="store_true",
                          help="get available databases")

        parser.add_option("--tables",
                          dest="getTables",
                          action="store_true",
                          help="get available databases "
                               "tables (optional: -D)")

        parser.add_option("--columns",
                          dest="getColumns",
                          action="store_true",
                          help="get table columns "
                               "(required: -T optional: -D)")

        parser.add_option("--dump",
                          dest="dumpTable",
                          action="store_true",
                          help="dump a database table "
                               "(required: -T optional: -D)")

        parser.add_option("--file",
                          dest="filename",
                          help="read a specific file content")

        parser.add_option("-e",
                          dest="expression",
                          help="expression to evaluate")

        parser.add_option("--union-check",
                          dest="unionCheck",
                          action="store_true",
                          help="check for UNION sql injection")

        parser.add_option("-D", "--database",
                          dest="db",
                          help="database to enumerate")

        parser.add_option("-T", "--table",
                          dest="tbl",
                          help="table to enumerate")

        parser.add_option("-v", "--verbose",
                          dest="verbose",
                          action="store_true",
                          help="print verbose output messages")

        (args, arg) = parser.parse_args()

        if not args.url:
            parser.error("missing url parameter (Hint: -h)")

        return args
    except (OptionError, TypeError), e:
        parser.error(e)


def getDbHandler(args):
    try:
        app = MySQLMap(args)
        if app.checkDbms():
            return app

        app = PostgreSQLMap(args)
        if app.checkDbms():
            return app

        app = MSSQLServerMap(args)
        if app.checkDbms():
            return app
    except:
        return None


def main():
    showBanner()
    print "[*] starting at: %s\n" % time.strftime("%X")

    app = None
    shellArgs = parseArgs()

    try:
        optObj = OptionCheck()
        checkedArgs = optObj.check(shellArgs)

        injObj = InjectionCheck()
        injArgs = injObj.check(checkedArgs)

        if not injArgs.injParameter:
            raise Exception, "all parameters are not injectable"

        app = getDbHandler(injArgs)

        if not app:
            errMsg  = "it is not possible to fingerprint "
            errMsg += "remote dbms"
            raise Exception, errMsg

        print "remote dbms:\t%s\n" % app.getFingerprint()
        dumper = SQLMapDump()

        if injArgs.getBanner:
            dumper.string("banner", app.getBanner())

        if injArgs.getCurrentUser:
            dumper.string("current user", app.getCurrentUser())

        if injArgs.getCurrentDb:
            dumper.string("current database", app.getCurrentDb())

        if injArgs.getUsers:
            dumper.list("database users", app.getUsers())

        if injArgs.getDbs:
            dumper.list("available databases", app.getDbs())

        if injArgs.getTables:
            dumper.dbTables(app.getTables())

        if injArgs.getColumns:
            dumper.dbTableColumns(app.getColumns())

        if injArgs.dumpTable:
            dumper.dbTableValues(app.dumpTable())

        if injArgs.filename:
            dumper.string(injArgs.filename, app.getFile(injArgs.filename))

        if injArgs.expression:
            dumper.string(injArgs.expression, app.getExpr(injArgs.expression))

        if injArgs.unionCheck:
            dumper.string("valid union", app.unionCheck())
    except Exception, e:
        print "[%s] [ERR] %s\n" % (time.strftime("%X"), str(e))

    print "[*] shutting down at: %s\n" % time.strftime("%X")


if __name__ == "__main__":
    main()
